require 'spec_helper.rb'

describe Temp do

  before :each do
    @dir = File.expand_path(File.dirname(__FILE__))
    @test_dir = File.join(@dir, 'test')
    @template_dir = File.join(@test_dir, 'template')

    FileUtils.mkdir_p(@template_dir)
    File.open(File.join(@test_dir, 'foo.txt'), 'w') { |f|
      f.write "Hello, World!\n" }

    @conf_file = File.join(@dir, 'tempconf.yml')
    @conf = Temp::Config.new(@conf_file)

    @template1_info = create_template(File.join(@template_dir, 'template1'),
                                 'Template 1',
                                 'A test template.',
                                 ['hello.txt',
                                  'ignore.txt',
                                  'foo',
                                  'foo/bar',
                                  'foo/bar/baz.txt'],
                                 ['**/*ignore*'],
                                 ['hello.txt'])
    @template1_hello_erb = 'Hello, <%= @foo %>!'
    @template1_hello = 'Hello, Foo!'
    File.open(File.join(@template1_info[:path], 'hello.txt'), 'w') { |f|
      f.write @template1_hello_erb }
    @template2_info = { :path => File.join(@template_dir, 'template2.txt'),
                   :name => 'template2.txt' }
    File.open(@template2_info[:path], 'w') { |f| f.write("Hello, World!\n") }
    @template1 = Temp::Template.new(@template1_info[:filename], @conf)
    @template2 = Temp::Template.new(@template2_info[:name], @conf)
  end

  after :each do
    FileUtils.rm_r(@test_dir)
  end

  describe Temp::Options do

    it 'should have the correct arguments' do
      Temp::Options.parse(['foo', 'bar'])[:args].should == ['foo', 'bar']
    end

    it 'should have the correct options' do
      options = Temp::Options.parse(['-f', '-bar-baz'])
      options[:f].should be_true
      options[:bar_baz].should be_true
    end

    it 'should set "no" options to false' do
      Temp::Options.parse(['-no-baz'])[:baz].should be_false
    end

    it 'should stop parsing options after a --' do
      options = Temp::Options.parse(['-foo', '--', '-bar'])
      options[:foo].should be_true
      options[:args].should == ['-bar']
    end

    it 'should stop parsing options after an option with --' do
      options = Temp::Options.parse(['--stop-hammertime', '-foo'])
      options[:args].should == ['-foo']
      options[:stop_hammertime].should be_true
    end

    it 'should set an option to a given value' do
      Temp::Options.parse(['-foo=bar'])[:foo].should == 'bar'
    end

  end

  describe Temp::Config do

    it 'should load the config file' do
      Temp::Config.new(@conf_file)
    end

    it 'should have the default template directory' do
      conf = Temp::Config.new('hopefully-nonexistent-file.txt')
      conf.template_dir.should == Temp::Config::DEFAULT_TEMP_DIR
    end

    it 'should have the template directory specified in the config file' do
      conf = Temp::Config.new(@conf_file)
      conf.template_dir.should == @template_dir
    end

  end

  describe Temp::Tempfile do

    describe 'template1 tempfile' do

      before :each do
        @tempfile = Temp::Tempfile.new(@template1_info[:path])
      end

      it 'should have the correct name' do
        @tempfile.info[:name].should == @template1_info[:name]
      end

      it 'should have the correct description' do
        @tempfile.info[:desc].should == @template1_info[:desc]
      end

      it 'should ignore certain files' do
        @tempfile.ignore_files.sort.should == @template1_info[:ignore]
      end

      it 'should use ERB on some files' do
        @tempfile.erb_files.should == @template1_info[:erb]
      end

    end

  end

  describe Temp::Template do

    it 'should return all templates' do
      Temp::Template.all(@conf).map { |t| t.name }.should ==
        [@template1_info[:name], @template2_info[:name]]
    end

    describe 'load template1' do

      before :each do
        @template = Temp::Template.new(@template1_info[:filename], @conf)
      end

      it 'should have correct name' do
        @template.name.should == @template1_info[:name]
      end

      it 'should have correct description' do
        @template.desc.should == @template1_info[:desc]
      end

      it 'should have the correct files' do
        @template.files.sort.should == @template1_info[:files] -
          @template1_info[:ignore]
      end

    end

    describe 'failure' do

      it 'should raise an exception when given a non-existing template name' do
        lambda do
          Temp::Template.new('foo', @conf)
        end.should raise_error Temp::Exceptions::TemplateNonExistentError
      end

    end

  end

  describe Temp::Project do

    before :each do
      @conf = Temp::Config.new(@conf_file)
    end

    describe 'initialization' do

      describe 'success' do

        describe 'template1 with non-existent path' do

          before :each do
            @name = 'foo'
            @path = File.join(@test_dir, @name)
            @proj = Temp::Project.new(@path, @template1_info)
          end

          it 'should have the correct name' do
            @proj.name.should == @name
          end

          it 'should have the correct path' do
            @proj.path.should == @path
          end

        end

        describe 'template1 with existing path' do

          before :each do
            @name = @template1_info[:filename]
            @path = File.join(@test_dir, @name)
            @proj = Temp::Project.new(@test_dir, @template1)
          end

          it 'should have the correct name' do
            @proj.name.should == @name
          end

          it 'should have the correct path' do
            @proj.path.should == @path
          end

        end

      end

      describe 'failure' do

        describe 'template1 with existing file as path' do

          it 'should raise an exception' do
            lambda do
              Temp::Project.new(File.join(@test_dir, 'foo.txt'),
                                @template1_info)
            end.should raise_error Temp::Exceptions::ProjectExistsError
          end

        end

        describe 'template1 with existing path and template in path' do

          it 'should raise an exception' do
            Dir.mkdir(File.join(@test_dir, @template1_info[:filename]))
            lambda do
              Temp::Project.new(@test_dir, @template1)
            end.should raise_error Temp::Exceptions::ProjectExistsError
          end

        end

      end

    end

    describe 'creation' do

      describe 'template1' do

        before :each do
          @proj_dir = File.join(@test_dir, 'project1')
          @proj = Temp::Project.new(@proj_dir, @template1)
        end

        it 'should have copied the correct files' do
          @proj.create
          Dir.glob(File.join(@proj_dir, '**/*')).map { |f|
            f.sub(@proj_dir + '/', '') }.sort.should ==
              @template1_info[:files] - @template1_info[:ignore]
        end

        it 'should use ERB on a file' do
          @proj.create
          File.read(File.join(@proj_dir, 'hello.txt')).should ==
            @template1_hello
        end

      end

      describe 'template2' do

        before :each do
          @proj_file = File.join(@test_dir, 'project.txt')
          @proj = Temp::Project.new(@proj_file, @template2)
        end

        it 'should have copied the template file' do
          @proj.create
          File.file?(@proj_file).should be_true
        end

      end

    end

  end

end
